//
//  SSA.m
//

#import "SSA.h"

@interface SSA ()
{
    UIImage * spritesheet_;
    NSTimer * timer_;

    CGRect      rect_;
    NSInteger   frameIndex_;
    NSInteger   counter_;
    NSInteger   countMax_;
    float       duration_;
    float       width_;
    float       height_;

    BOOL        isAnimation_;
    BOOL        isPause_;

    SSAGetImage getImage_;
    SSAComplete complete_;
}
@end

@implementation SSA


#pragma mark - --------------------------------------------------------------------------
#pragma mark - public

- (id)initWithSetSSImage:(UIImage *)spritesheet duration:(float)duration clipRect:(CGRect)clipRect
{
    self = [super init];
    if (self)
    {
        counter_        = 0;
        frameIndex_     = 0;
        duration_       = duration;
        rect_           = clipRect;
        width_          = rect_.size.width;
        height_         = rect_.size.height;
        spritesheet_    = spritesheet;
        countMax_       = spritesheet_.size.width / width_;

        isAnimation_    = NO;
        isPause_        = NO;
        _isLoop         = NO;
    }
    return self;
}

- (void)startAnimationWithGetImage:(SSAGetImage)getImage complete:(SSAComplete)complete
{
    if (isAnimation_) return;
    isAnimation_ = YES;

    if (frameIndex_ != 0) [self resetWithCounter:NO];
    else [self resetWithCounter:YES];

    getImage_ = getImage;
    complete_ = complete;
    [self setImage];
    [self timer];
}

- (void)pause
{
    if (isPause_) return;
    isPause_ = YES;
    [timer_ invalidate];
}

- (void)resume
{
    if (!isPause_) return;
    isPause_ = NO;
   [self timer];
}

- (void)stop
{
    if (!isAnimation_) return;
    isAnimation_ = NO;

    [self resetWithCounter:YES];
    UIImage * ssImage = [self getSSImage:counter_];
    getImage_(ssImage);
    complete_(NO, _isLoop);
}

- (UIImage *)getImageByFrame:(NSInteger)index setCounter:(BOOL)setCounter
{
    frameIndex_ = index;
    if (setCounter)
    {
        counter_ = index;
    }
    return [self getSSImage:frameIndex_];
}

- (BOOL)isAnimation
{
    return isAnimation_;
}

- (BOOL)isPause
{
    return isPause_;
}


#pragma mark - --------------------------------------------------------------------------
#pragma mark - private

- (void)timer
{
    timer_ = [NSTimer scheduledTimerWithTimeInterval:duration_ target:self selector:@selector(setImage) userInfo:nil repeats:YES];
}

- (void)setImage
{
    UIImage * ssImage = [self getSSImage:counter_];
    counter_ ++;

    // call blocks
    getImage_(ssImage);

    // complete
    if(counter_ == countMax_)
    {
        [self resetWithCounter:YES];
        if(_isLoop)
        {
            isAnimation_ = YES;
            [self timer];
        }
        else
        {
            UIImage * ssImage = [self getSSImage:counter_];
            getImage_(ssImage);
        }
        complete_(YES, _isLoop);
    }
}

- (void)resetWithCounter:(BOOL)flag
{
    isPause_ = NO;
    isAnimation_ = NO;
    [timer_ invalidate];
    if (flag) counter_ = 0;
}

- (UIImage *)getSSImage:(NSInteger)index
{
    float scale = [[UIScreen mainScreen] scale];
    CGRect clipRect = CGRectMake( (index * (width_)) * scale, 0, width_ * scale, height_ * scale);
    UIImage * ssImage = [UIImage imageWithCGImage:CGImageCreateWithImageInRect( [spritesheet_ CGImage] , clipRect )];
    return ssImage;
}


@end
